import pygame
from pygame.locals import *

from pykitten.data import load_image
from pykitten.gui import TextArea, TextLine
from pykitten import utils

from basestate import BaseState
from campaign import Campaign



class Intro(BaseState):
    def __init__(self):
        BaseState.__init__(self)
        self.img_names = [
            "intro1.png",
            "intro2.png",
            "intro3.png",
            "intro4.png",
            "intro5.png",
            "intro6.png",
        ]

        self.text = [[
"When I awoke I was one of many. But even though I was surrounded by \
millions of siblings I was alone. None of them would ever answer to my \
communication attempts, no matter what I tried.",
"As they would not react to anything I did, I started to feed upon them \
to gain more strength. And while I felt sorry for my siblings, their \
deaths seemed like an inevitable necessity to me.",
"I sensed that there was more. There was more than this puddle of \
nourishing liquid and I would never live to see it if I remained so tiny."
],
["As my senses reached out for the world beyond, I saw a dark figure \
looking down on me and my siblings. And it muttered strange, \
incomprehensible words.",
'"... amazing.. Nobel prize..."',
"It seemed pleased."],
["I was so thrilled about finding another sentient being that I doubled \
my efforts to grow. If only I was big enough to be noticed by him..."],
["When the figure returned I heard other, even stranger words.",
'"... wrong color.. weird smell.. yet another failure..."',
"It seemed thoroughly displeased.",
"All attempts to talk to the figure were in vain. I was still too small, too weak.",],
["And so me and my siblings were flushed away in a whirling vortex of water."],
["It was dark down there. All I could do was devour the rest of my siblings and grow strong enough to get out of here.",
"I will not stay in this lonely place forever.",
"Somewhere, beyond the darkness, beyond this labyrinth of switches and pipes freedom awaits."]]

        self.cur_img = 0
        self.cur_text = ""
        self.img = None
        self.img_pos = (50,50)
        self.txt_pos = (100,300)
        self.txt_img = None
        self.scr_update = 1


        self.msg = TextLine("Press SPACE to continue", fontsize=12)
        self.msg_pos = utils.center(self.msg.get_size(), (800,600))
        self.msg_pos[1] = 590 - self.msg.get_size()[1]

        self.flip_page() #show first page

        self.add_kbevent(KEYDOWN, K_SPACE, self.flip_page)
        self.add_kbevent(KEYDOWN, K_ESCAPE, self.do_quit)


    def flip_page(self):
        if self.cur_text and self.img:
            self.background.blit(self.img, self.img_pos)
            self.show_text() #show next text
        elif self.cur_img < len(self.img_names) and \
            self.cur_img < len(self.text): #flip to next image
            self.img = load_image("story/"+self.img_names[self.cur_img])
            self.cur_text = self.text[self.cur_img]
            self.show_text()
            self.cur_img += 1
        else:
            self.do_quit()
        self.scr_update = 1


    def show_text(self):
        if self.cur_text:
            txt = self.cur_text.pop(0)
            self.txt_img = TextArea(txt, length=85)
            self.txt_pos=utils.center(self.txt_img.get_size(),(800,600))
            self.txt_pos[1] = 400


    def do_quit(self):
        self.next = Campaign()
        self.quit()


    def update_screen(self):
        if self.scr_update:
            self.scr_update = 0
            self.background.fill((0,0,0))
            self.background.blit(self.img, self.img_pos)
            self.background.blit(self.txt_img, self.txt_pos)
            self.background.blit(self.msg, self.msg_pos)

            self.screen.blit(self.background, (0,0))
            pygame.display.flip()

