import pygame
from pygame.locals import *

from pykitten.gui import ImgButton

from widgets import TextButton, WobbleButton, FlipButton
from resources import *
from basestate import BaseState
from game import CustomGame
from campaign import Campaign
from map import draw_preview


class MapSelect(BaseState):
    def __init__(self, folder="maps"):
        BaseState.__init__(self)
        self.scr_update = 1
        self.fps = 30

        self.wobbles = []
        self.fg = GFX_bg["levelselect.png"]

        self.selected = None
        self.folder = folder
        self.map_list = get_mapnames(folder)

        if len(self.map_list) == 0:
            self.quit()
            return

        self.filebtn_pos =  [  #file button positions
            (19,80),
            (18,134),
            (18,186),
            (18,239),
            (18,292),
            (18,344),
        ]

        #maplist
        #self.file_surf = pygame.Surface((280,310))
        #self.file_surf.fill((0,0,0))
        self.file_pos = (22,85)
        #self.background.blit(self.file_surf, self.file_pos)

        self.high_layer = pygame.Surface((800,600))
        self.high_layer.set_colorkey((255,0,255))
        self.high_layer.fill((255,0,255))

        #file buttons
        self.cur_page = 0
        self.max_lines = 6
        offset = 52

        img = pygame.Surface((235,30))
        img.fill((10,10,10))
        x = self.file_pos[0] + 15
        y = self.file_pos[1] + 13 -5
        self.file_btn = []
        for i in xrange(self.max_lines):
            btn = TextButton(img, callback=self.select_map, nr=i)
            btn.set_pos((x, y))
            self.file_btn.append(btn)
            self.widgets.append(btn)
            y += offset

        self.draw_filenames()

        #next/prev page buttons
        img2 = GFX_menu["next.png"]

        img1 = pygame.Surface(img2.get_size())
        img1.set_colorkey((255,0,255))
        img1.fill((255,0,255))

        btn = FlipButton(img1, img2, callback=self.scroll, dir=1)
        btn.set_pos((306, 254))
        btn.draw(self.background)
        self.wobbles.append(btn)

        img2 = GFX_menu["prev.png"]

        btn = FlipButton(img1, img2, callback=self.scroll, dir=-1)
        btn.set_pos((305, 148))
        btn.draw(self.background)
        self.wobbles.append(btn)

        #map preview
        self.prev_surf = pygame.Surface((400,300))
        self.prev_surf.fill((60,200,60))
        self.prev_pos = (363,85)

        self.select_map(0) #select first map

        #start map button
        img1 = GFX_menu["select1.png"]
        img2 = GFX_menu["select2.png"]
        btn = WobbleButton(img1, img2, callback=self.start_map)
        pos = (750 - (btn.get_size()[0]), 500)
        btn.set_pos(pos)
        #btn.draw(self.background)
        self.wobbles.append(btn)

        #quit map selection button
        img1 = GFX_menu["back1.png"]
        img2 = GFX_menu["back2.png"]
        btn = WobbleButton(img1, img2, callback=self.quit)
        btn.set_pos((50, 500))
        #btn.draw(self.background)
        self.wobbles.append(btn)

        #keyboard shortcuts
        self.add_kbevent(KEYDOWN, K_ESCAPE, self.quit)


    def scroll(self, dir):
        if self.cur_page+dir < 0 or \
            (self.cur_page+dir) * self.max_lines >= len(self.map_list):
            return

        self.cur_page += dir
        self.draw_filenames()
        self.select_map(0)


    def select_map(self, nr):
        #highight selected map, gen preview
        if nr + (self.cur_page * self.max_lines) >= len(self.map_list):
            return
        btn_new = self.file_btn[nr]

        #de-highlight old selection
        self.high_layer.fill((0,0,0))

        if self.selected is not None:
            btn_old = self.file_btn[self.selected]
            btn_old.unhighlight()
        self.high_layer.fill((255,0,255))

        #highlight new selection
        btn_new = self.file_btn[nr]
        btn_new.highlight()
        self.high_layer.blit(GFX_filebg[nr], self.filebtn_pos[nr])

        self.selected = nr
        self.scr_update = 1

        mapdata = load_mapdata(self.file_btn[nr].text, self.folder)
        draw_preview(mapdata, self.prev_surf)
        self.high_layer.blit(self.prev_surf, self.prev_pos)


    def start_map(self):
        nr = self.selected
        btn = self.file_btn[nr]
        map_name = btn.text
        mapdata = load_mapdata(map_name, self.folder)
        if mapdata:
            self.next = CustomGame(mapdata)
            self.pause()
            self.quit()


    def unpause(self):
        self.draw_filenames()
        BaseState.unpause(self)


    def draw_filenames(self):
        start = self.cur_page * self.max_lines

        for i in xrange(self.max_lines):
            text = ""
            if start + i < len(self.map_list):
                text = self.map_list[start+i]
            self.file_btn[i].set_text(text)
            self.file_btn[i].draw(self.background)

        self.scr_update = 1


    def update_screen(self):
        self.background.blit(GFX_bg["levelselect.png"], (0,0))
        self.background.blit(self.high_layer, (0,0))
        for w in self.wobbles:
            w.update()
            w.draw(self.background)
        self.draw_filenames()
        self.screen.blit(self.background, (0,0))

        pygame.display.flip()


class ContMapSelect(MapSelect):
    def __init__(self):
        MapSelect.__init__(self, folder="maps")
        cur = SAVE["cur_level"]
        self.map_list = self.map_list[:cur+1]
        self.draw_filenames()

    def start_map(self):
        nr = self.selected + (self.cur_page * self.max_lines)
        if nr < len(self.map_list):
            self.next = Campaign(nr)
            self.quit()
